package es.uca.xmleye.eventos;

/**
 * Reúne toda la información acerca de una petición de búsqueda a lo largo de un
 * documento. No contiene información acerca del documento en sí.
 * 
 * Se permite usar rutas XPath, de forma excluyente al resto de las búsquedas.
 */
public class EventoBusqueda implements Cloneable {
	private String claveBusqueda;
	private boolean bPalabraCompleta, bCoincidenMayusMinus;
	private AlcanceBusqueda alcance;

	private String xpath;

	/**
	 * Accesor a la clave de búsqueda.
	 * 
	 * @return Clave de búsqueda.
	 */
	public String getClaveBusqueda() {
		return claveBusqueda;
	}

	/**
	 * Accesor a la opción "palabras completas".
	 * 
	 * @return <code>true</code> si está activa, <code>false</code> si no.
	 */
	public boolean getPalabraCompleta() {
		return bPalabraCompleta;
	}

	/**
	 * Accesor a la opción "coincidir mayúsculas/minúsculas".
	 * 
	 * @return <code>true</code> si está activa, <code>false</code> si no.
	 */
	public boolean getCoincidenMayusMinus() {
		return bCoincidenMayusMinus;
	}

	/**
	 * Accesor al alcance de la búsqueda.
	 * 
	 * @return Alcance que debe tener la búsqueda desde su contexto.
	 */
	public AlcanceBusqueda getAlcance() {
		return alcance;
	}

	/**
	 * Accesor a la ruta XPath.
	 * 
	 * @return Ruta XPath a usar directamente (excluye a todas las demás
	 *         opciones).
	 */
	public String getRutaXPath() {
		return xpath;
	}

	/**
	 * Mutador de la clave de búsqueda.
	 * 
	 * @param s
	 *            Nueva clave de búsqueda.
	 */
	public void setClaveBusqueda(String s) {
		claveBusqueda = s;
	}

	/**
	 * Mutador de la opción "palabras completas".
	 * 
	 * @param b
	 *            <code>true</code> la activa, <code>false</code> la
	 *            desactiva.
	 */
	public void setPalabraCompleta(boolean b) {
		bPalabraCompleta = b;
	}

	/**
	 * Mutador de la opción "coincidir mayúsculas/minúsculas".
	 * 
	 * @param b
	 *            <code>true</code> la activa, <code>false</code> la
	 *            desactiva.
	 */
	public void setCoincidenMayusMinus(boolean b) {
		bCoincidenMayusMinus = b;
	}

	/**
	 * Mutador del alcance de búsqueda.
	 * 
	 * @param f
	 *            Nuevo alcance de búsqueda.
	 */
	public void setAlcance(AlcanceBusqueda f) {
		alcance = f;
	}

	/**
	 * Mutador de la ruta XPath.
	 * 
	 * @param xpath
	 *            Nueva ruta XPath.
	 */
	public void setRutaXPath(String ruta) {
		xpath = ruta;
	}

	public EventoBusqueda clone() {
		try {
			return (EventoBusqueda) super.clone();
		} catch (CloneNotSupportedException ex) {
			return null;
		}
	}

	public String toString() {
		return "Cadena: " + claveBusqueda
				+ (bPalabraCompleta ? ", palabras completas " : "")
				+ (bCoincidenMayusMinus ? ", considerando mayúsculas" : "")
				+ ", " + alcance
				+ (xpath != null ? (", con ruta " + xpath) : "");
	}

	public boolean equals(Object o) {
		if (!(o instanceof EventoBusqueda) || o == null)
			return false;

		final EventoBusqueda e = (EventoBusqueda) o;
		return this == e
				|| (e != null 
						&& e.claveBusqueda != null
						&& e.claveBusqueda.equals(claveBusqueda)
						&& e.bPalabraCompleta == bPalabraCompleta
						&& e.bCoincidenMayusMinus == bCoincidenMayusMinus
						&& e.alcance != null 
						&& e.alcance.equals(alcance) 
						&& (e.xpath == null && e.xpath == xpath
							|| e.xpath != null && e.xpath.equals(xpath)));
	}
}
