package es.uca.xmleye.prefs;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.io.*;
import java.util.*;

/**
 * <p>
 * Implementación de GestorPreferencias basada en varios HashMap serializados.
 * Están envueltos con funciones que evitan accesos concurrentes múltiples por
 * varios hilos a los mismos datos.
 * </p>
 * 
 * <p>
 * Se recomienda (por claridad) un esquema de nombrado de atributos parecido a
 * las variables de Perl:
 * </p>
 * <ul>
 * <li>una opción escalar empieza por $</li>
 * <li>una opción lista empieza por @</li>
 * <li>una opción hash empieza por %</li>
 * </li>
 */
class GestorPreferenciasSerializado implements Serializable, GestorPreferencias {
	/**
	 * 
	 */
	private static final long serialVersionUID = 2584515580341639223L;
	private String fichPreferencias;
	private HashMap<String, String> datosEscalares;
	private HashMap<String, List<String>> datosListas;

	/**
	 * Crea un nuevo <code>GestorPreferenciasSerializado</code>, preparándolo
	 * para las sucesivas invocaciones de <code>cargar</code> y
	 * <code>guardar</code>.
	 * 
	 * @param ruta
	 *            Ruta relativa desde el CWD al fichero.
	 */
	GestorPreferenciasSerializado(String ruta) {
		this.fichPreferencias = ruta;
		this.datosEscalares = new HashMap<String, String>();
		this.datosListas = new HashMap<String, List<String>>();
	}

	/**
	 * Carga los datos almacenados del fichero con el
	 * <code>GestorPreferenciasSerializado</code> serializado anteriormente.
	 * 
	 * @throws ExcepcionPreferencias
	 *             Error de E/S al cargar los datos, o el fichero especificado
	 *             en el constructor no tiene un formato válido.
	 */
	public synchronized void cargar() throws ExcepcionPreferencias {
		try {
			cargar(fichPreferencias);
		} catch (Exception ex) {
			throw new ExcepcionPreferencias(ex);
		}
	}

	/**
	 * Guarda los datos almacenados en este momento en el fichero especificado
	 * al crear el objeto, reemplazando el fichero original.
	 * 
	 * @throws ExcepcionPreferencias
	 *             Error de E/S al guardar los datos.
	 */
	public synchronized void guardar() throws ExcepcionPreferencias {
		try {
			guardar(fichPreferencias);
		} catch (Exception ex) {
			throw new ExcepcionPreferencias(ex);
		}
	}

	public void setPrefEscalar(String clave, String escalar) {
		synchronized (datosEscalares) {
			datosEscalares.put(clave, escalar);
		}
	}

	public String getPrefEscalar(String clave) {
		synchronized (datosEscalares) {
			return datosEscalares.get(clave);
		}
	}

	public String getPrefEscalar(String clave, String omision) {
		synchronized (datosEscalares) {
			String valor = datosEscalares.get(clave);

			if (valor == null) {
				setPrefEscalar(clave, omision);
				return omision;
			}
			return valor;
		}
	}

	public void setPrefLista(String clave, List<String> lista) {
		synchronized (datosListas) {
			datosListas.put(clave, lista);
		}
	}

	public List<String> getPrefLista(String clave) {
		synchronized (datosListas) {
			return datosListas.get(clave);
		}
	}

	public List<String> getPrefLista(String clave, List<String> omision) {
		synchronized (datosListas) {
			List<String> valor = datosListas.get(clave);
			if (valor == null) {
				setPrefLista(clave, omision);
				return omision;
			}
			return valor;
		}
	}

	private void cargar(String ruta) throws IOException, ClassNotFoundException {
		File f = new File(ruta);
		if (!f.exists()) {
			// El fichero no existe - usamos las opciones por defecto
			return;
		}

		InputStream fIS = new FileInputStream(f);
		InputStream bIS = new BufferedInputStream(fIS);
		ObjectInputStream oIS = new ObjectInputStream(bIS);

		GestorPreferenciasSerializado res = (GestorPreferenciasSerializado) oIS
				.readObject();
		oIS.close();

		this.datosListas = res.datosListas;
		this.datosEscalares = res.datosEscalares;
	}

	private void guardar(String ruta) throws IOException {
		File f = new File(ruta);

		OutputStream fIS = new FileOutputStream(f);
		OutputStream bIS = new BufferedOutputStream(fIS);
		ObjectOutputStream oIS = new ObjectOutputStream(bIS);

		oIS.writeObject(this);
		oIS.flush();
		oIS.close();
	}

}
