package es.uca.xmleye.tipos;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.io.*;

import java.util.HashMap;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Map;

import org.w3c.dom.*;
import org.xml.sax.*;
import javax.xml.parsers.*;
import javax.xml.xpath.*;

import es.uca.xmleye.xpath.HashNamespaceContext;

/**
 * Tipo de documento para el que se tiene soporte.
 * 
 * En las órdenes, se utiliza <code>%s</code> para indicar el fichero fuente.
 * Se sustituirá con la ruta completa correcta cuando sea necesario.
 * 
 * Estos documentos son ficheros XML sencillos, del formato:
 * 
 * <pre>
 *  &lt;![CDATA[
 *  &lt;type&gt;
 *  &lt;!-- Nombre por defecto --&gt;
 *  &lt;name&gt;Mi Tipo&lt;/name&gt;
 *  &lt;!-- Nombre en inglés (Estados Unidos) --&gt;
 *  &lt;name language=&quot;en_US&quot;&gt;My Type&lt;/name&gt;
 *  &lt;edit_cmd&gt;mi-editor-favorito %s&lt;/edit_cmd&gt;
 *  &lt;import_cmd&gt;mi-preproc-favorito %s&lt;/import_cmd&gt;
 *  &lt;extensions&gt;
 *  &lt;extension&gt;abc&lt;/extension&gt;
 *  &lt;extension&gt;def&lt;/extension&gt;
 *  &lt;/extensions&gt;
 *  &lt;/type&gt;
 *  ]]&gt;
 * </pre>
 * 
 * Esta clase provee los métodos necesarios para acceder a dicha información de
 * manera directa, sin que sea necesario manipular el código XML.
 * 
 * Para más detalles, consultar la DTD <code>URI_DTD</code> del directorio
 * <code>src/tipos</code>.
 * 
 * La orden de importación es opcional, cubriendo así los casos en que
 * trabajemos directamente con ficheros XML.
 * 
 * @author Antonio García Domínguez
 * @version 1.0
 */
public class Tipo {
	final protected static String TIPO_NS_PREFIX = "d";
	final protected static String TIPO_NS_URI = "http://xmleye.uca.es/xmleye/accepted-doc";

	final protected static String TAG_TIPO = "type";
	final protected static String TAG_NOMBRE = "name";
	final protected static String TAG_ORDEN_EDICION = "edit_cmd";
	final protected static String TAG_ORDEN_IMPORTACION = "import_cmd";
	final protected static String TAG_EXTENSIONES = "extensions";
	final protected static String TAG_EXTENSION = "extension";

	final protected static String ATTR_IDIOMA = "language";

	/**
	 * Ruta relativa a la DTD en la fuente de menor prioridad (normalmente la
	 * del sistema).
	 */
	final protected static String URI_DTD = "accepted-doc.dtd";

	// Usado para internacionalizar los mensajes de error
	protected static final ResourceBundle errRes;

	static {
		errRes = ResourceBundle.getBundle("es.uca.xmleye.l10n.ErrorMessages",
				Locale.getDefault());
	}

	private File[] fuentes;
	private String ordenEdicion;
	private String ordenImportacion;
	private String[] extensiones;

	// Los nombres son localizables por país e idioma
	private Map<String, String> mapNombres;
	// Se notifica al padre cuando cambia el nombre
	private RepositorioTipos padre;

	/**
	 * Construye el tipo cargándolo desde un fichero.
	 * 
	 * Método de conveniencia para <code>Tipo(File[])</code>.
	 * 
	 * @param padre
	 *            Repositorio en que se halla contenido este tipo.
	 * @param fuente
	 *            Fichero desde el que cargar los datos.
	 * @return Tipo con los datos cargados.
	 * @exception ExcepcionTipo
	 *                No se pudieron cargar los datos.
	 */
	Tipo(RepositorioTipos padre, File fuente) throws ExcepcionTipo {
		cargar(padre, new File[] { fuente });
	}

	/**
	 * Construye el tipo intentando cargarlo de un conjunto de ficheros.
	 * 
	 * @param padre
	 *            Repositorio en que se halla contenido este tipo.
	 * @param fuentes
	 *            Lista de posibles fuentes en orden descendente de prioridad.
	 * @return Tipo con los datos cargados.
	 * @exception ExcepcionTipo
	 *                No se pudieron cargar los datos.
	 */
	Tipo(RepositorioTipos padre, File[] fuentes) throws ExcepcionTipo {
		cargar(padre, fuentes);
	}

	/** METODOS PUBLICOS * */

	/** * ACCESORES ** */
	public String getOrdenEdicion() {
		return ordenEdicion;
	}

	public String getOrdenImportacion() {
		return ordenImportacion;
	}

	public String getNombre() {
		final String idioma = Locale.getDefault().getLanguage();
		final String pais = Locale.getDefault().getCountry();

		// Claves a ir probando
		final String[] claves = new String[] { Locale.getDefault().toString(),
				idioma + "_" + pais, idioma };

		for (String k : claves) {
			if (mapNombres.containsKey(k))
				return mapNombres.get(k);
		}
		return mapNombres.get("");
	}

	public String[] getExtensiones() {
		return extensiones;
	}

	/** * MUTADORES ** */

	public void setOrdenEdicion(String orden) {
		this.ordenEdicion = orden;
	}

	public void setOrdenImportacion(String orden) {
		this.ordenImportacion = orden;
	}

	public void setNombre(String nombre) {
		// Usamos siempre la coincidencia más exacta
		mapNombres.put(Locale.getDefault().toString(), nombre);
		padre.nombreCambiado(this);
	}

	public void setExtensiones(String[] exts) {
		this.extensiones = exts;
	}

	/** * CONVERSIÓN A CADENA ** */
	public String toString() {
		return getNombre();
	}

	/**
	 * Retira una personalización del tipo, sin llegar a borrarlo.
	 * 
	 * El proceso consiste simplemente en borrar el primer fichero de tipo en
	 * prioridad, tras comprobar que existe un segundo, y volver a cargar los
	 * datos.
	 * 
	 * @exception ExcepcionTipo
	 *                No se ha podido eliminar la versión personalizada.
	 */
	public void restaurar() throws ExcepcionTipo {
		try {
			int posPrimero;
			int posSegundo;

			// Buscamos la versión segunda en prioridad
			for (posPrimero = 0; posPrimero < fuentes.length
					&& !fuentes[posPrimero].canRead(); ++posPrimero)
				;

			for (posSegundo = posPrimero + 1; posSegundo < fuentes.length
					&& !fuentes[posSegundo].canRead(); ++posSegundo)
				;

			// Existe: borramos la más importante
			if (posSegundo < fuentes.length) {
				fuentes[posPrimero].delete();
			}

			// Cargamos de nuevo los datos
			cargar(padre, fuentes);
		} catch (SecurityException ex) {
			throw new ExcepcionTipo(ex);
		}
	}

	/** METODOS DE PAQUETE * */

	/**
	 * Carga desde la posible fuente útil con más prioridad.
	 * 
	 * @param padre
	 *            Repositorio padre al que notificar cambios en el nombre.
	 * @param fuentes
	 *            Lista de posibles ficheros fuente, ordenados en prioridad
	 *            descendente.
	 * @exception ExcepcionTipo
	 *                Hubo algún problema al cargar.
	 */
	void cargar(RepositorioTipos padre, File[] fuentes) throws ExcepcionTipo {
		this.padre = padre;
		this.fuentes = fuentes;

		for (File f : fuentes) {
			if (f.canRead()) {
				cargar(f);
				return;
			}
		}

		throw new ExcepcionTipo(errRes.getString("err_cannot_open_type"));
	}

	/**
	 * Carga los datos desde el fichero <code>fuente</code>.
	 * 
	 * El fichero <code>fuente</code> pasará a usarse en el próximo guardado.
	 * 
	 * @param fuente
	 *            Fichero desde el que cargar los datos.
	 * @exception ExcepcionTipo
	 *                Hubo algún problema al cargar.
	 */
	void cargar(File fuente) throws ExcepcionTipo {
		try {
			// Usaremos un parser que valide la entrada
			DocumentBuilderFactory docBF = DocumentBuilderFactory.newInstance();
			docBF.setNamespaceAware(true);
			docBF.setValidating(false);

			// Analizamos la entrada
			DocumentBuilder docB = docBF.newDocumentBuilder();
			Document doc = docB.parse(fuente);

			// Emplearemos XPath para extraer la información que
			// necesitamos (hay que ajustar el nombre de espacios)
			XPath xpath = XPathFactory.newInstance().newXPath();
			HashNamespaceContext nsContext = new HashNamespaceContext();
			nsContext.set(TIPO_NS_PREFIX, TIPO_NS_URI);
			xpath.setNamespaceContext(nsContext);

			this.mapNombres = new HashMap<String, String>();
			NodeList listaNombres = (NodeList) xpath.evaluate(String.format(
					"/p:%s/p:%s".replaceAll("p(?=:)", TIPO_NS_PREFIX),
					TAG_TIPO, TAG_NOMBRE), doc, XPathConstants.NODESET);
			for (int i = 0; i < listaNombres.getLength(); ++i) {
				Element elem = (Element) listaNombres.item(i);
				String idioma = elem.getAttribute(ATTR_IDIOMA);
				mapNombres.put(idioma != null ? idioma : "", elem
						.getFirstChild().getNodeValue());
			}

			this.ordenEdicion = (String) xpath.evaluate(String.format(
					"/p:%s/p:%s/text()".replaceAll("p(?=:)", TIPO_NS_PREFIX),
					TAG_TIPO, TAG_ORDEN_EDICION), doc);
			this.ordenImportacion = (String) xpath.evaluate(String.format(
					"/p:%s/p:%s/text()".replaceAll("p(?=:)", TIPO_NS_PREFIX),
					TAG_TIPO, TAG_ORDEN_IMPORTACION), doc);
			NodeList listaExts = (NodeList) xpath.evaluate(String.format(
					"/p:%s/p:%s/p:%s/text()".replaceAll("p(?=:)",
							TIPO_NS_PREFIX), TAG_TIPO, TAG_EXTENSIONES,
					TAG_EXTENSION), doc, XPathConstants.NODESET);

			this.extensiones = new String[listaExts.getLength()];
			for (int i = 0; i < listaExts.getLength(); ++i) {
				this.extensiones[i] = listaExts.item(i).getNodeValue();
			}

		} catch (IOException ex) {
			throw new ExcepcionTipo(ex);
		} catch (SAXException ex) {
			throw new ExcepcionTipo(ex);
		} catch (ParserConfigurationException ex) {
			throw new ExcepcionTipo(ex);
		} catch (XPathExpressionException ex) {
			throw new ExcepcionTipo(ex);
		}
	}

	/**
	 * Guarda en la fuente original con más prioridad.
	 * 
	 * @exception ExcepcionTipo
	 *                Hubo algún problema al guardar.
	 */
	void guardar() throws ExcepcionTipo {
		guardar(fuentes);
	}

	/**
	 * Guarda en el fichero con más prioridad escribible de
	 * <code>destinos</code>.
	 * 
	 * @param destinos
	 *            Lista de posibles destinos ordenada descendentemente por
	 *            prioridad.
	 * @exception ExcepcionTipo
	 *                Hubo algún problema al guardar.
	 */
	void guardar(File[] destinos) throws ExcepcionTipo {
		try {
			for (File f : destinos) {
				if (f.canWrite() || f.createNewFile()) {
					guardar(f, destinos[destinos.length - 1].getAbsoluteFile()
							.getParent());
					return;
				}
			}
		} catch (IOException ex) {
			throw new ExcepcionTipo(ex);
		}

		throw new ExcepcionTipo(errRes.getString("err_cannot_save_type"));
	}

	/**
	 * Guarda los cambios en el fichero <code>destino</code>.
	 * 
	 * @param destino
	 *            Fichero al que guardar los datos.
	 * @param dirDTD
	 *            Directorio en el que se halla la DTD.
	 * @exception ExcepcionTipo
	 *                Hubo algún problema al guardar.
	 */
	void guardar(File destino, final String dirDTD) throws ExcepcionTipo {
		try {
			PrintWriter ow = new PrintWriter(new OutputStreamWriter(
					new BufferedOutputStream(new FileOutputStream(destino)),
					"UTF-8"));

			ow.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
			ow.println(String.format("<!DOCTYPE %s SYSTEM \"%s/%s\">",
					TAG_TIPO, dirDTD, URI_DTD));
			ow.println(String
					.format("<%s xmlns=\"%s\">", TAG_TIPO, TIPO_NS_URI));

			for (Map.Entry<String, String> elem : mapNombres.entrySet()) {
				if ("".equals(elem.getKey())) {
					ow.println(String.format("  <%s>%s</%s>", TAG_NOMBRE, elem
							.getValue(), TAG_NOMBRE));
				} else {
					ow.println(String.format("  <%s language=\"%s\">%s</%s>",
							TAG_NOMBRE, elem.getKey(), elem.getValue(),
							TAG_NOMBRE));
				}
			}

			ow.println(String.format("  <%s>%s</%s>", TAG_ORDEN_EDICION,
					getOrdenEdicion(), TAG_ORDEN_EDICION));
			ow.println(String.format("  <%s>%s</%s>", TAG_ORDEN_IMPORTACION,
					getOrdenImportacion(), TAG_ORDEN_IMPORTACION));
			ow.println(String.format("  <%s>", TAG_EXTENSIONES));
			for (String ext : getExtensiones()) {
				ow.println(String.format("    <%s>%s</%s>", TAG_EXTENSION, ext,
						TAG_EXTENSION));
			}
			ow.println(String.format("  </%s>", TAG_EXTENSIONES));
			ow.println(String.format("</%s>", TAG_TIPO));

			ow.close();
		} catch (FileNotFoundException ex) {
			throw new ExcepcionTipo(ex);
		} catch (UnsupportedEncodingException ex) {
			throw new ExcepcionTipo(ex);
		}
	}

}