package es.uca.xmleye.util;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Clase de utilidad usada para destacar una determinada cadena dentro de un
 * documento XHTML.
 */
public class MarcadorClaveXHTML {

	/**
	 * Destaca las ocurrencias de la clave de búsqueda en el código fuente XHTML
	 * <code>texto</code>. Se preservan los tags y todo aquello que no esté
	 * dentro del elemento <code>&lt;body&gt;</code>.
	 * 
	 * @param texto
	 *            Código fuente XHTML.
	 * @param clave
	 *            Clave a reemplazar dentro del texto XHTML.
	 * @param regexpRepl
	 *            Expresión regular a usar para el reemplazo (<code>$1</code>
	 *            es la clave original).
	 * @param bPalabraCompleta
	 *            <code>true</code> si se desean sólo ocurrencias con la
	 *            palabra completa (si no es una palabra, con espacio en blanco
	 *            o fin de línea alrededor), <code>false</code> de lo
	 *            contrario.
	 * @param bCoincidenMayusMinus
	 *            <code>true</code> si se desean sólo ocurrencias que
	 *            coincidan en mayúsculas y minúsculas, <code>false</code> de
	 *            lo contrario.
	 * 
	 * @return Código fuente XHTML con toda ocurrencia de la clave
	 *         <code>clave</code> reemplazada mediante la expresión regular
	 *         <code>regexpRepl</code>.
	 */
	public String marcarApariciones(String texto, String clave,
			String regexpRepl, boolean bPalabraCompleta,
			boolean bCoincidenMayusMinus) {
		
		if (clave == null) return texto;

		/** * CONSTRUCCIÓN REGEXP ************************* */

		// Escapamos la clave de búsqueda antes de usarla en la regexp
		clave = clave.replaceAll("([{}\\[\\]()^$.|*+?\\\\])", "\\\\$1");

		// Comenzamos a construir la expresión regular a usar
		String regexp = clave;

		// Palabras completas
		if (bPalabraCompleta) {
			final String delimInicio = Character.isLetterOrDigit(clave
					.charAt(0)) ? "\\b" : "(?<=\\s|^|>)";
			final String delimFin = Character.isLetterOrDigit(clave
					.charAt(clave.length() - 1)) ? "\\b" : "(?=\\s|$|<)";

			regexp = delimInicio + regexp + delimFin;
		}

		/*
		 * Evitamos que reemplace en el interior de los tags.
		 * 
		 * Ejemplo:
		 * 
		 * <foo><hola>hola</hola></foo>
		 * 
		 * "foo" al principio tiene > antes de < - no coincide (tag) "hola"
		 * dentro del tag tiene un > antes del < - no coincide (tag) "hola" en
		 * el texto no tiene > antes del siguiente < - coincide (nodo texto)
		 * "hola" en el segundo tag no tiene < a continuación - no coincide
		 * (tag, final documento)
		 * 
		 */
		regexp = "(" + regexp + ")" + "(?=[^>]*?<.*?>)";

		/** * REEMPLAZO *********************************** */

		// Creamos el Matcher
		final Matcher matcher = Pattern.compile(regexp,
				(bCoincidenMayusMinus ? 0 : Pattern.CASE_INSENSITIVE)).matcher(
				texto);
		// Limitamos su región para evitar reemplazar en la cabecera
		matcher.region(texto.indexOf("<body>") + 4, texto.indexOf("</body>"));

		// Acumulamos los reemplazos y el final
		final StringBuffer resultado = new StringBuffer();
		while (matcher.find()) {
			matcher.appendReplacement(resultado, regexpRepl);
		}
		matcher.appendTail(resultado);

		return resultado.toString();
	}

}
