package es.uca.xmleye.xpath;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import org.w3c.dom.*;
import javax.xml.xpath.*;

/**
 * Clase para obtención de rutas XPath a partir de nodos de un árbol DOM y
 * viceversa.
 */
public final class GestorRutasXPath {
	private static XPath xpath;

	/**
	 * Obtiene una ruta unívoca XPath a un nodo desde una raíz. El único nodo no
	 * accesible del árbol original de esta forma es el DOCTYPE, y es por diseño
	 * del propio XPath (sencillamente no forma parte del modelo de datos que
	 * usa).
	 * 
	 * @see <a href="http://www.w3.org/TR/xpath#data-modelo">XPath Data Model</a>
	 * 
	 * @param nodo
	 *            Nodo destino.
	 * @param raiz
	 *            Nodo origen.
	 * @return Cadena con la expresion XPath necesaria, o <code>null</code> si
	 *         es imposible acceder a dicho nodo desde la raíz usando XPath.
	 * @throws NullPointerException
	 *             El nodo no es descendiente de la raiz.
	 */
	public static String obtenerRuta(Node nodo, Node raiz)
			throws NullPointerException {
		// Ruta vacia
		if (nodo.equals(raiz))
			return ".";

		// Como minimo necesitaremos un salto ahora
		StringBuffer ruta = new StringBuffer("");
		while (!nodo.equals(raiz)) {
			final Node padre = nodo.getParentNode();
			final NodeList hermanos = padre.getChildNodes();

			// Buscamos el numero de secuencia del nodo en su arbol
			// (empezando por 1), excluyendo el DOCTYPE (no forma parte del
			// modelo del árbol usado por XPath)
			int indice = 0;
			for (int i = 0; i < hermanos.getLength()
					&& !hermanos.item(i).equals(nodo); ++i) {
				if (hermanos.item(i).getNodeType() != Node.DOCUMENT_TYPE_NODE) {
					++indice;
				}
			}

			// Agregamos por el principio este paso
			ruta.insert(0, "/node()[" + (indice + 1) + "]");

			// Nos movemos hacia arriba
			nodo = padre;
		}

		return ruta.toString();
	}

	/**
	 * Obtiene los nodos referidos por la ruta XPath <code>ruta</code> bajo el
	 * contexto del nodo <code>raíz</code>.
	 * 
	 * @param ruta
	 *            Ruta XPath a seguir. Debe identificar a un nodo de forma
	 *            unívoca.
	 * @param raiz
	 *            Nodo que actúa como contexto inicial de XPath.
	 * @return Nodos referidos por la ruta XPath.
	 * @throws XPathExpressionException
	 *             La ruta XPath en <code>ruta</code> no es válida.
	 */
	public static NodeList obtenerNodo(String ruta, Node raiz)
			throws XPathExpressionException {
		if (xpath == null) {
			xpath = XPathFactory.newInstance().newXPath();
		}

		final NodeList nodos = (NodeList) xpath.evaluate(ruta, raiz,
				XPathConstants.NODESET);
		return nodos;
	}

}
