package es.uca.xmleye.tipos;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import junit.framework.*;
import junit.textui.*;

import java.io.File;
import java.util.*;

import es.uca.xmleye.tipos.ExcepcionTipo;
import es.uca.xmleye.tipos.RepositorioTipos;
import es.uca.xmleye.tipos.Tipo;

public class PruebaRepositorioTipos extends TestCase {
	final private static String REPO_DIR     = "build/types";
	final private static String REPO_DIR_ALT = "build/types3";
	final private static String TEST_TYPE    = "xml.type";

	private RepositorioTipos repositorio;

	/** Crea un nuevo caso de prueba. */
	public PruebaRepositorioTipos(String nombre) {
		super(nombre);
	}

	/** Crea el conjunto de pruebas para JUnit. */
	public static Test suite() {
		return new TestSuite(PruebaRepositorioTipos.class);
	}

	/** Lanza el caso de prueba mediante JUnit. */
	public static void main(String[] args) {
		TestRunner.run(suite());
	}

	/** Probamos que el repositorio se abre correctamente. */
	public void testApertura() {
		assertEquals(1, repositorio.getSize());

		final Tipo tipoXML = (Tipo) repositorio.getElementAt(0);
		assertEquals(tipoXML, repositorio.getTipos().get(0));
		assertEquals("Fichero XML normal", tipoXML.getNombre());
		assertEquals("emacs %s", tipoXML.getOrdenEdicion());
		
		assertTrue("No hay orden de importación para el tipo por defecto", 
				tipoXML.getOrdenImportacion().length()==0);
		assertEqualElems(new String[] { "xml" }, tipoXML
				.getExtensiones());
	}

	/** Probamos que los datos se guardan bien. */
	public void testGuardado() throws ExcepcionTipo {
		final Tipo tipoXML = (Tipo) repositorio.getElementAt(0);
		tipoXML.setNombre("prueba");
		repositorio.guardar();

		// Comprobamos que ha cambiado el fichero
		repositorio = new RepositorioTipos(new String[] { REPO_DIR_ALT,
				REPO_DIR });
		final Tipo tipoXMLb = (Tipo) repositorio.getElementAt(0);
		assertEquals("prueba", tipoXMLb.getNombre());
		assertEquals("emacs %s", tipoXML.getOrdenEdicion());
		assertTrue("No hay orden de importación para el tipo por defecto", 
				tipoXML.getOrdenImportacion().length()==0);
		assertEqualElems(new String[] { "xml" }, tipoXML
				.getExtensiones());

		// Lo ponemos como estaba
		tipoXMLb.restaurar();
		repositorio.guardar();
	}

	/** Probamos que los nombres son localizables y se guardan bien. */
	public void testLocalizacion() throws ExcepcionTipo {
		final Tipo tipoXML = (Tipo) repositorio.getElementAt(0);

		// Activamos la local española y probamos (hacemos cambios)
		Locale.setDefault(new Locale("es"));
		assertEquals("Fichero XML normal", tipoXML.getNombre());
		tipoXML.setNombre("Prueba1");

		// Activamos la local inglesa y probamos (hacemos cambios)
		Locale.setDefault(new Locale("en"));
		assertEquals("Regular XML file", tipoXML.getNombre());
		tipoXML.setNombre("Prueba2");

		// Guardamos y volvemos a intentarlo
		repositorio.guardar();
		repositorio = new RepositorioTipos(new String[] { REPO_DIR_ALT,
				REPO_DIR });
		final Tipo tipoXMLb = (Tipo) repositorio.getElementAt(0);

		// Activamos la local española y probamos
		Locale.setDefault(new Locale("es"));
		assertEquals("Prueba1", tipoXMLb.getNombre());

		// Activamos la local inglesa y probamos
		Locale.setDefault(new Locale("en"));
		assertEquals("Prueba2", tipoXMLb.getNombre());

		tipoXML.restaurar();
		repositorio.guardar();
	}

	public void testMultiDirs() throws Exception {
		// Tomamos la fecha del fichero xml.type original
		final long modAntes = new File(REPO_DIR + File.separator + TEST_TYPE)
				.lastModified();

		final Tipo tipoXML = (Tipo) repositorio.getElementAt(0);
		tipoXML.setNombre("prueba");
		repositorio.guardar();

		// El fichero original no debe haber cambiado
		final long modDespues = new File(REPO_DIR + File.separator + TEST_TYPE)
				.lastModified();
		assertEquals("Fichero con menos prioridad no ha cambiado", modAntes,
				modDespues);

		// Los cambios han de haberse guardado en el directorio con más
		// prioridad
		final long modPrioritario = new File(REPO_DIR_ALT + File.separator
				+ TEST_TYPE).lastModified();
		assertTrue("Cambios guardados en el de más prioridad",
				modPrioritario > modAntes);

		tipoXML.restaurar();
		repositorio.guardar();
	}

	public void testRestaurar() throws Exception {
		// Cambiamos y guardamos
		final Tipo tipoXML = (Tipo) repositorio.getElementAt(0);
		String nombreAntiguo = tipoXML.getNombre();
		tipoXML.setNombre("prueba_restaurar");
		repositorio.guardar();

		// Comprobamos que se ha cambiado, restauramos y guardamos
		repositorio = new RepositorioTipos(new String[] { REPO_DIR_ALT,
				REPO_DIR });
		final Tipo tipoXMLb = (Tipo) repositorio.getElementAt(0);
		assertEquals("prueba_restaurar", tipoXMLb.getNombre());
		tipoXMLb.restaurar();
		repositorio.guardar();

		// Volvemos a abrir y comprobamos que vuelve a estar como antes
		repositorio = new RepositorioTipos(new String[] { REPO_DIR_ALT,
				REPO_DIR });
		final Tipo tipoXMLc = (Tipo) repositorio.getElementAt(0);
		assertEquals(nombreAntiguo, tipoXMLc.getNombre());
	}

	public void testBusqueda() throws Exception {
		// Ahora pedimos por tipo
		final Tipo tipoXML = repositorio.getTipoPorRuta("probando.xml");
		assertMember("La búsqueda de un tipo que existe funciona", "xml",
				tipoXML.getExtensiones());

		// Comprobamos que también falla sin problemas
		final Tipo tipoFalso = repositorio.getTipoPorRuta("probando.noexisto");
		assertNull("La búsqueda de un tipo que no existe devuelve null",
				tipoFalso);
	}

	protected void assertEqualElems(String[] vA, String[] vB) {
		assertEquals(vA.length, vB.length);
		for (int i = 0; i < vA.length; ++i) {
			assertEquals(vA[i], vB[i]);
		}
	}

	protected void assertMember(String msg, String aguja, String[] pajar) {
		for (String elem : pajar) {
			if (aguja.equals(elem))
				return;
		}

		fail(msg);
	}

	public void setUp() throws ExcepcionTipo {
		repositorio = new RepositorioTipos(new String[] { REPO_DIR_ALT,
				REPO_DIR });
	}

	public void tearDown() {
		repositorio = null;
	}

}